# Tenant Policy Configuration (Version: SM 5.17.0)

Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.

## Overview

Tenant configuration policies are managed by cluster system management packages.

Some service may provides bare-metal provisioning of NVIDIA cloud resources.  This allows users to configure the bare-metal to suit their particular requirements.

Since the user has control of the hardware (as root), they can configure the system
to include any virtual ports they choose.  The user can possibly configure their hardware to 
subvert the cloud.

The tenants validation function is intended to stop this type of action.
 

## OpenSM Tenants configuration file

Tenants policies are configured via the *tenants_policy_file* parameter in the OpenSM
configuration file.  For example:
```
	tenants_policy_file /etc/opensm/conf/local_tenants_file
```
The default may be also be changed at run time by using the --tenants_policy_file option with OpenSM.


## Flow Overview 

Virtual port may be managed via system services.  Characteristics of the managed virtual ports are sent to UFM.
UFM will communicate the characteristics via the tenants_policy_conf file.  UFM writes the file and then sends a SIGHUP to SM.  The SIGHUP signal trigger a new parsing of the file.  When parsing is complete, the virt-mgr will revalidate all of the current virtual ports to ensure that there are no violations.

The virt-mgr also calls the validation functions upon the creation of any virtual port.

Currently, the tenant manager is only responsible for those virtual ports which it reads from the tenants_policy_conf file.

There may be other virtual ports running in the fabric.  For any given &lt;phys-guid,virt-guid&gt; pair there are 5 possibilities:

|  case   | virtual   |  physical  | same group |  status     |  reason                     |
|---------|-----------|------------|------------|-------------|-----------------------------|
| case 1  | managed   | managed    |    yes     |  good       |  both ports in same group   |
| case 2  | managed   | managed    |    no      |  violation  |  ports in different groups  |
| case 3  | managed   | unmanaged  |    n/a     |  violation  |  only one port managed      |
| case 4  | unmanaged | managed    |    n/a     |  violation  |  only one port managed      |
| case 5  | unmanaged | unmanaged  |    n/a     |  good       |  neither port managed       |


Case 1 validates the &lt;phys-guid,virt-guid&gt; tuple as being paired in the policy file.

Case 2 invalidates the pair because they are not in the same group. (See below for a 
full example.)

Case 3 and 4 detail the cases when one of the guids is managed and the other is not.
In these cases, the pair is invalidated.

Case 5 validates any pair in which neither guid is managed.


## File Format

### Comments:

Line content followed an '#' character is a comment and ignored by the parser.

White space is ignored by the parser, so the file can be formatted for human
consumption.


### General file format:
```
	map-group
	name: <uint64_t>
		physical-port-guid: <list of guids separated by commas>
		virtual-port-guid: <list of guids separated by commas>
	end-map-group
```

*map-group* is a literal string which denotes the beginning of a group of 
resources.

*name:* is a literal string followed by a non-zero uint64_t.  This identifier
is used as the value mapped to the guids which follow.

*physical-port-guid:* is a literal string followed by a comma separated list of
guids in this map.

*virtual-port-guid:* is a literal string followed by a comma separated list of
virtual guids in this map.

*end-map-group* is a literal string ending the map group definition.


### Example:

```
	map-group
	name: 3
		physical-port-guid: 0x0002c9000000000a
		virtual-port-guid: 0x0002c9000000000a, 0x0002c90000000045, 0x0002c90000000038, 0x0002c9000000003a
	end-map-group

	map-group

	name: 9
		physical-port-guid: 0x0002c90000000006
		virtual-port-guid: 0x0002c90000000041, 0x0002c90000000095, 0x0002c90000000049, 0x0002c9000000004b
	end-map-group
```


In group '3', there is one physical port whose guid is 0x0002c9000000000a.

The virtual ports on this physical port can have one of the guids listed.

For example, 0x0002c90000000045 would be appropriate, but 0x0002c90000077777 would not because it is not in the list.


## GUID Validation

The virtual to physial guid validation occurs via the virt-mgr.  It calls osm_tenant_validate() with the guids in question.  The tenant-mgr queries the guid database (which is derived from the policy file) and determines the validity of the pairing.  In the event that the pairing is not valid, the virtual port is destroyed.

In the examples above,  
   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;osm_tenant_validate(0x0002c90000000045, 0x0002c9000000000a) succeeds - the guids are in the same group  
   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;osm_tenant_validate(0x0002c90000077777, 0x0002c9000000000a) fails - the virtual guid is not in the same group

## UFM Interaction

UFM writes the policy file and triggers a rescan via the SIGHUP mechanism.

In the event of a failed validation, SM sends a trap 1601 to UFM.
